@@ -38,16 +38,39 @@
 #include "gbf-project.h"
 #include "gbf-project-model.h"
 #include "gbf-project-util.h"
-#include "eggtreemodelfilter.h"
 
 
 #define GLADE_FILE GBF_GLADEDIR "/create_dialogs.glade"
 
+static GtkWidget *
+custom_widget_handler (GladeXML *xml,
+                       gchar    *func_name,
+                       gchar    *name,
+                       gchar    *string1,
+                       gchar    *string2,
+                       gint      int1,
+                       gint      int2,
+                       gpointer  user_data)
+{
+    GtkWidget *wid = NULL;
+
+    if (!strcmp (string1, "GtkComboBox")) {
+        wid = gtk_combo_box_new ();
+    } else if (!strcmp (string1, "GbfProjectView")) {
+        wid = gbf_project_view_new ();
+    } else {
+        g_warning ("Unknown custom widget type '%s'", string1);
+    }
+
+    return wid;
+}
+
 static GladeXML *
 load_interface (const gchar *top_widget)
 {
     GladeXML *xml;
 
+    glade_set_custom_handler (custom_widget_handler, NULL);
     xml = glade_xml_new (GLADE_FILE, top_widget, GETTEXT_PACKAGE);
 
     if (!xml) {
@@ -84,8 +107,8 @@ setup_groups_treeview (GbfProjectModel *model,
     g_return_if_fail (model != NULL);
     g_return_if_fail (view != NULL && GBF_IS_PROJECT_VIEW (view));
 
-    filter = egg_tree_model_filter_new (GTK_TREE_MODEL (model), NULL);
-    egg_tree_model_filter_set_visible_func (EGG_TREE_MODEL_FILTER (filter),
+    filter = gtk_tree_model_filter_new (GTK_TREE_MODEL (model), NULL);
+    gtk_tree_model_filter_set_visible_func (GTK_TREE_MODEL_FILTER (filter),
 					    groups_filter_fn, NULL, NULL);
     gtk_tree_view_set_model (GTK_TREE_VIEW (view), GTK_TREE_MODEL (filter));
     g_object_unref (filter);
@@ -96,8 +119,8 @@ setup_groups_treeview (GbfProjectModel *model,
                                                    GBF_TREE_NODE_GROUP, select_group)) {
         GtkTreeIter iter_filter;
         
-        egg_tree_model_filter_convert_child_iter_to_iter (
-            EGG_TREE_MODEL_FILTER (filter), &iter_filter, &iter);
+        gtk_tree_model_filter_convert_child_iter_to_iter (
+            GTK_TREE_MODEL_FILTER (filter), &iter_filter, &iter);
         path = gtk_tree_model_get_path (filter, &iter_filter);
 
     } else {
@@ -105,8 +128,8 @@ setup_groups_treeview (GbfProjectModel *model,
 
         root_path = gbf_project_model_get_project_root (model);
         if (root_path) {
-            path = egg_tree_model_filter_convert_child_path_to_path (
-                EGG_TREE_MODEL_FILTER (filter), root_path);
+            path = gtk_tree_model_filter_convert_child_path_to_path (
+                GTK_TREE_MODEL_FILTER (filter), root_path);
             gtk_tree_path_free (root_path);
         } else {
             path = gtk_tree_path_new_first ();
@@ -165,7 +188,7 @@ gbf_project_util_new_group (GbfProjectModel *model,
 			    const gchar     *default_group)
 {
     GladeXML *gui;
-    GtkWidget *dialog, *groups_ph, *group_name_entry, *ok_button;
+    GtkWidget *dialog, *group_name_entry, *ok_button;
     GtkWidget *groups_view;
     gint response;
     GbfProject *project;
@@ -182,7 +205,7 @@ gbf_project_util_new_group (GbfProjectModel *model,
     
     /* get all needed widgets */
     dialog = glade_xml_get_widget (gui, "new_group_dialog");
-    groups_ph = glade_xml_get_widget (gui, "groups_ph");
+    groups_view = glade_xml_get_widget (gui, "groups_view");
     group_name_entry = glade_xml_get_widget (gui, "group_name_entry");
     ok_button = glade_xml_get_widget (gui, "ok_button");
     
@@ -191,10 +214,8 @@ gbf_project_util_new_group (GbfProjectModel *model,
                       (GCallback) entry_changed_cb, ok_button);
     gtk_widget_set_sensitive (ok_button, FALSE);
     
-    groups_view = gbf_project_view_new ();
-    gtk_widget_show (groups_view);
-    gtk_container_add (GTK_CONTAINER (groups_ph), groups_view);
     setup_groups_treeview (model, groups_view, default_group);
+    gtk_widget_show (groups_view);
     
     if (parent) {
 	gtk_window_set_transient_for (GTK_WINDOW (dialog), parent);
@@ -254,58 +275,54 @@ gbf_project_util_new_group (GbfProjectModel *model,
     g_object_unref (gui);
 }
 
-static GtkWidget *
-build_types_menu (GbfProject *project)
+enum {
+    TARGET_TYPE_TYPE = 0,
+    TARGET_TYPE_NAME,
+    TARGET_TYPE_PIXBUF,
+    TARGET_TYPE_N_COLUMNS
+};
+
+/* create a tree model with the target types */
+static GtkListStore *
+build_types_store (GbfProject *project)
 {
-    GtkWidget *menu;
+    GtkListStore *store;
+    GtkTreeIter iter;
     gchar **types;
     gint i;
     GdlIcons *icons;
     
     types = gbf_project_get_types (project);
-    menu = gtk_menu_new ();
-    g_object_set_data_full (G_OBJECT (menu), "types-stringv",
-                            types, (GDestroyNotify) g_strfreev);
-    icons = gdl_icons_new (24, 16.0);
+    store = gtk_list_store_new (TARGET_TYPE_N_COLUMNS,
+                                G_TYPE_STRING,
+                                G_TYPE_STRING,
+                                GDK_TYPE_PIXBUF);
+
+    icons = gdl_icons_new (16);
     
     for (i = 0; types [i]; i++) {
-        GtkWidget *item, *image = NULL;
         GdkPixbuf *pixbuf;
         const gchar *name;
 
         name = gbf_project_name_for_type (project, types [i]);
         pixbuf = gdl_icons_get_mime_icon (icons,
                                           gbf_project_mimetype_for_type (project, types [i]));
-        item = gtk_image_menu_item_new_with_label (name);
-        if (pixbuf) {
-            image = gtk_image_new_from_pixbuf (pixbuf);
-            gtk_widget_show (image);
-            gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
+
+        gtk_list_store_append (store, &iter);
+        gtk_list_store_set (store, &iter,
+                            TARGET_TYPE_TYPE, types [i],
+                            TARGET_TYPE_NAME, name,
+                            TARGET_TYPE_PIXBUF, pixbuf,
+                            -1);
+
+        if (pixbuf)
             g_object_unref (pixbuf);
-        }
-        gtk_widget_show (item);
-        gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
     }
 
     g_object_unref (icons);
-    
-    return menu;
-}
+    g_strfreev (types);
 
-static void 
-type_changed_cb (GtkWidget *widget, gpointer user_data)
-{
-    GtkWidget *menu;
-    gchar **types, *selected_type;
-    
-    g_return_if_fail (GTK_IS_OPTION_MENU (widget));
-    menu = gtk_option_menu_get_menu (GTK_OPTION_MENU (widget));
-
-    types = g_object_get_data (G_OBJECT (menu), "types-stringv");
-    selected_type = types [gtk_option_menu_get_history (
-                               GTK_OPTION_MENU (widget))];
-    g_object_set_data_full (G_OBJECT (widget), "type",
-                            g_strdup (selected_type), (GDestroyNotify) g_free);
+    return store;
 }
 
 void 
@@ -314,8 +331,10 @@ gbf_project_util_new_target (GbfProjectModel *model,
 			     const gchar     *default_group)
 {
     GladeXML *gui;
-    GtkWidget *dialog, *groups_ph, *target_name_entry, *ok_button;
-    GtkWidget *target_type_optionmenu, *groups_view, *types_menu;
+    GtkWidget *dialog, *target_name_entry, *ok_button;
+    GtkWidget *target_type_combo, *groups_view;
+    GtkListStore *types_store;
+    GtkCellRenderer *renderer;
     gint response;
     GbfProject *project;
     gboolean finished = FALSE;
@@ -331,10 +350,9 @@ gbf_project_util_new_target (GbfProjectModel *model,
     
     /* get all needed widgets */
     dialog = glade_xml_get_widget (gui, "new_target_dialog");
-    groups_ph = glade_xml_get_widget (gui, "groups_ph");
+    groups_view = glade_xml_get_widget (gui, "groups_view");
     target_name_entry = glade_xml_get_widget (gui, "target_name_entry");
-    target_type_optionmenu =
-        glade_xml_get_widget (gui, "target_type_optionmenu");
+    target_type_combo = glade_xml_get_widget (gui, "target_type_combo");
     ok_button = glade_xml_get_widget (gui, "ok_button");
     
     /* set up dialog */
@@ -342,20 +360,35 @@ gbf_project_util_new_target (GbfProjectModel *model,
                       (GCallback) entry_changed_cb, ok_button);
     gtk_widget_set_sensitive (ok_button, FALSE);
     
-    groups_view = gbf_project_view_new ();
-    gtk_widget_show (groups_view);
-    gtk_container_add (GTK_CONTAINER (groups_ph), groups_view);
     setup_groups_treeview (model, groups_view, default_group);
+    gtk_widget_show (groups_view);
+
+    /* setup target types combo box */
+    types_store = build_types_store (project);
+    gtk_combo_box_set_model (GTK_COMBO_BOX (target_type_combo), 
+                             GTK_TREE_MODEL (types_store));
+
+    /* create cell renderers */
+    renderer = gtk_cell_renderer_pixbuf_new ();
+    gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (target_type_combo),
+                                renderer, FALSE);
+    gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (target_type_combo),
+                                    renderer,
+                                    "pixbuf", TARGET_TYPE_PIXBUF,
+                                    NULL);
+
+    renderer = gtk_cell_renderer_text_new ();
+    gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (target_type_combo),
+                                renderer, TRUE);
+    gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (target_type_combo),
+                                    renderer,
+                                    "text", TARGET_TYPE_NAME,
+                                    NULL);
+    gtk_widget_show (target_type_combo);
+
+    /* preselect */
+    gtk_combo_box_set_active (GTK_COMBO_BOX (target_type_combo), 0);
 
-    types_menu = build_types_menu (project);
-    gtk_option_menu_set_menu (GTK_OPTION_MENU (target_type_optionmenu),
-                              types_menu);
-    g_signal_connect (target_type_optionmenu, "changed",
-                      (GCallback) type_changed_cb, NULL);
-    gtk_option_menu_set_history (GTK_OPTION_MENU (target_type_optionmenu), 0);
-    /* force data setting */
-    type_changed_cb (target_type_optionmenu, NULL);
-    
     if (parent) {
 	gtk_window_set_transient_for (GTK_WINDOW (dialog), parent);
     }
@@ -369,15 +402,22 @@ gbf_project_util_new_target (GbfProjectModel *model,
             {
                 GError *err = NULL;
                 GbfTreeData *data;
-                gchar *group_id = NULL, *name, *type;
+                GtkTreeIter iter;
+                gchar *group_id = NULL, *name, *type = NULL;
                 
                 name = gtk_editable_get_chars (
                     GTK_EDITABLE (target_name_entry), 0, -1);
                 data = gbf_project_view_find_selected (GBF_PROJECT_VIEW (groups_view),
                                                        GBF_TREE_NODE_GROUP);
-                type = g_object_get_data (G_OBJECT (target_type_optionmenu), "type");
+
+                /* retrieve target type */
+                if (gtk_combo_box_get_active_iter (GTK_COMBO_BOX (target_type_combo), &iter)) {
+                    gtk_tree_model_get (GTK_TREE_MODEL (types_store), &iter, 
+                                        TARGET_TYPE_TYPE, &type,
+                                        -1);
+                }
                 
-                if (data) {
+                if (data && type) {
                     gchar *new_target = NULL;
                     
                     gtk_widget_hide (dialog);
@@ -394,6 +434,7 @@ gbf_project_util_new_target (GbfProjectModel *model,
             
                     g_free (new_target);
                     g_free (group_id);
+                    g_free (type);
                     finished = TRUE;
                     
                 } else {
@@ -412,7 +453,7 @@ gbf_project_util_new_target (GbfProjectModel *model,
     }
     
     /* destroy stuff */
-    gtk_option_menu_remove_menu (GTK_OPTION_MENU (target_type_optionmenu));
+    g_object_unref (types_store);
     gtk_widget_destroy (dialog);
     g_object_unref (gui);
 }
@@ -444,8 +485,8 @@ setup_targets_treeview (GbfProjectModel *model,
     g_return_if_fail (model != NULL);
     g_return_if_fail (view != NULL && GBF_IS_PROJECT_VIEW (view));
 
-    filter = egg_tree_model_filter_new (GTK_TREE_MODEL (model), NULL);
-    egg_tree_model_filter_set_visible_func (EGG_TREE_MODEL_FILTER (filter),
+    filter = gtk_tree_model_filter_new (GTK_TREE_MODEL (model), NULL);
+    gtk_tree_model_filter_set_visible_func (GTK_TREE_MODEL_FILTER (filter),
 					    targets_filter_fn, NULL, NULL);
     gtk_tree_view_set_model (GTK_TREE_VIEW (view), GTK_TREE_MODEL (filter));
     g_object_unref (filter);
@@ -457,8 +498,8 @@ setup_targets_treeview (GbfProjectModel *model,
                                        GBF_TREE_NODE_TARGET, select_target)) {
             GtkTreePath *path;
             
-            egg_tree_model_filter_convert_child_iter_to_iter (
-                EGG_TREE_MODEL_FILTER (filter), &iter_filter, &iter);
+            gtk_tree_model_filter_convert_child_iter_to_iter (
+                GTK_TREE_MODEL_FILTER (filter), &iter_filter, &iter);
             path = gtk_tree_model_get_path (filter, &iter_filter);
             gtk_tree_view_set_cursor (GTK_TREE_VIEW (view), path, NULL, FALSE);
             gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (view), path, NULL, TRUE, 0.5, 0.0);
@@ -503,7 +544,7 @@ gbf_project_util_add_source (GbfProjectModel *model,
 			     const gchar     *default_uri)
 {
     GladeXML *gui;
-    GtkWidget *dialog, *targets_ph, *source_file_entry;
+    GtkWidget *dialog, *source_file_entry;
     GtkWidget *ok_button, *browse_button;
     GtkWidget *targets_view;
     gint response;
@@ -522,7 +563,7 @@ gbf_project_util_add_source (GbfProjectModel *model,
     
     /* get all needed widgets */
     dialog = glade_xml_get_widget (gui, "add_source_dialog");
-    targets_ph = glade_xml_get_widget (gui, "targets_ph");
+    targets_view = glade_xml_get_widget (gui, "targets_view");
     source_file_entry = glade_xml_get_widget (gui, "source_file_entry");
     browse_button = glade_xml_get_widget (gui, "browse_button");
     ok_button = glade_xml_get_widget (gui, "ok_button");
@@ -542,10 +583,8 @@ gbf_project_util_add_source (GbfProjectModel *model,
     g_object_set_data_full (G_OBJECT (browse_button), "root",
                             project_root, g_free);
     
-    targets_view = gbf_project_view_new ();
-    gtk_widget_show (targets_view);
-    gtk_container_add (GTK_CONTAINER (targets_ph), targets_view);
     setup_targets_treeview (model, targets_view, default_target);
+    gtk_widget_show (targets_view);
     
     if (parent) {
 	gtk_window_set_transient_for (GTK_WINDOW (dialog), parent);
